@@ -4,6 +4,7 @@ use 5.010001;
 use strict;
 use warnings;
 
+use Cwd;
 use List::MoreUtils qw(uniq);
 
 our %SPEC;
@@ -11,8 +12,10 @@ require Exporter;
 our @ISA       = qw(Exporter);
 our @EXPORT_OK = qw(complete_module);
 
-our $VERSION = '0.02'; # VERSION
-our $DATE = '2014-06-24'; # DATE
+our $VERSION = '0.03'; # VERSION
+our $DATE = '2014-06-25'; # DATE
+
+our @_built_prefix;
 
 $SPEC{complete_module} = {
     v => 1.1,
@@ -37,11 +40,10 @@ _
             req => 1,
             pos => 0,
         },
-        # a bit cumbersome to support on case-sensitive fs
-        #ci => {
-        #    summary => 'Whether to do case-insensitive search',
-        #    schema  => 'bool*',
-        #},
+        ci => {
+            summary => 'Whether to do case-insensitive search',
+            schema  => 'bool*',
+        },
         separator => {
             schema  => 'str*',
             default => '::',
@@ -79,7 +81,7 @@ sub complete_module {
     my %args = @_;
 
     my $word = $args{word} // '';
-    #my $ci   = $args{ci};
+    my $ci   = $args{ci};
     my $sep  = $args{separator} // '::';
 
     my $find_pm      = $args{find_pm}     // 1;
@@ -89,49 +91,96 @@ sub complete_module {
 
     my $sep_re = qr!(?:::|/|\Q$sep\E)!;
 
-    my ($prefixes, $pm);
+    my ($prefix0, $pm);
     if ($word =~ m!(.+)$sep_re(.*)!) {
         $word = $2;
-        $prefixes = [split($sep_re, $1)];
+        $prefix0 = [split($sep_re, $1)];
     } else {
-        $prefixes = [];
+        $prefix0 = [];
         $pm = $word;
     }
-    my $prefix = join("/", @$prefixes);
-    #say "D:prefixes=[".join(",",@$prefixes)."] prefix=$prefix word=$word";
-    my $resprefix = $prefix . (length($prefix) ? $sep : '');
+    my $prefix = join "/", @$prefix0;
+    #say "D:prefix0=[".join(",",@$prefix0)."] prefix=$prefix word=$word";
 
-    my $word_re = qr/\A\Q$word/;
+    my $word_re = $ci ? qr/\A\Q$word/i : qr/\A\Q$word/;
+
+    my @dirs;
+    if ($ci && @$prefix0) {
+        # for case-insensitive search: for each prefix we'll need to handle the
+        # possibility of more than one matches (e.g. foo/ and Foo/ on a
+        # case-sensitive filesystem)
+        my $cwd = getcwd();
+        my $j;
+        for my $dir (@INC) {
+            #say "D:dir=$dir";
+            next if ref($dir);
+            chdir $cwd if $j++;
+            chdir $dir or next;
+            my $dig;
+            local @_built_prefix;
+            $dig = sub {
+                my $i = shift;
+                #say "D:  i=$i, built_prefix=".join("/", @_built_prefix).", cwd=".getcwd();
+                opendir my($dh), "." or return;
+                for my $e (readdir $dh) {
+                    next if $e eq '.' || $e eq '..';
+                    next unless (-d $e) && lc($e) eq lc($prefix0->[$i]);
+                    local $_built_prefix[$i] = $e;
+                    if ($i == @$prefix0-1) {
+                        push @dirs, [
+                            $dir . (@_built_prefix ? "/" : "") .
+                                join("/", @_built_prefix),
+                            join($sep, @_built_prefix) .
+                                (@_built_prefix ? $sep:''),
+                        ];
+                    } else {
+                        chdir $e or return;
+                        $dig->($i+1);
+                        chdir "..";
+                    }
+                }
+            };
+            $dig->(0);
+        }
+        chdir $cwd;
+    } else {
+        for my $dir0 (@INC) {
+            next if ref($dir0);
+            my $dir = $dir0 . (length($prefix) ? "/$prefix" : "");
+            next unless -d $dir;
+            push @dirs, [$dir, join($sep, @$prefix0) . (@$prefix0 ? $sep:'')];
+        }
+    }
 
     my @res;
-    for my $dir (@INC) {
+    for my $e (@dirs) {
+        my ($dir, $resprefix) = @$e;
         next if ref($dir);
-        my $diro = $dir . (length($prefix) ? "/$prefix" : "");
-        #say "D:diro=$diro";
-        opendir my($dh), $diro or next;
+        opendir my($dh), $dir or next;
         for my $e (readdir $dh) {
             #say "D:$dir <$e>";
             next if $e =~ /\A\.\.?/;
-            my $is_dir = (-d "$diro/$e"); # stat once
-            #say "  D:<$e> is dir" if $is_dir;
+            my $is_dir = (-d "$dir/$e"); # stat once
+            #say "D:  <$e> is dir" if $is_dir;
             if ($find_prefix && $is_dir) {
-                #say "  D:<$e> $word_re";
+                #say "D:  <$e> $word_re";
                 push @res, $resprefix . $e . $sep if $e =~ $word_re;
             }
             my $f;
-            if ($find_pm && $e =~ /(.+)\.pm\z/) {
+            if ($find_pm && $e =~ qr/(.+)\.pm\z/) {
                 $f = $1;
                 push @res, $resprefix . $f if $f =~ $word_re;
             }
-            if ($find_pmc && $e =~ /(.+)\.pmc\z/) {
+            if ($find_pmc && $e =~ qr/(.+)\.pmc\z/) {
                 $f = $1;
                 push @res, $resprefix . $f if $f =~ $word_re;
             }
-            if ($find_pod && $e =~ /(.+)\.pod\z/) {
+            if ($find_pod && $e =~ qr/(.+)\.pod\z/) {
                 $f = $1;
                 push @res, $resprefix . $f if $f =~ $word_re;
             }
         }
+
     }
 
     [sort(uniq(@res))];
@@ -152,7 +201,7 @@ Complete::Module - Complete Perl module names
 
 =head1 VERSION
 
-This document describes version 0.02 of Complete::Module (from Perl distribution Complete-Module), released on 2014-06-24.
+This document describes version 0.03 of Complete::Module (from Perl distribution Complete-Module), released on 2014-06-25.
 
 =head1 SYNOPSIS
 
@@ -182,6 +231,10 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
+=item * B<ci> => I<bool>
+
+Whether to do case-insensitive search.
+
 =item * B<find_pm> => I<bool> (default: 1)
 
 Whether to find .pm files.
